//**********************************************************************
// Demo Application for simple USB to DMX512 Convertor based on the 
// FTDI Chip FT232BM. The device is controlled via FTDI's Direct Drivers
// 12.2003 by Stefan Kalbermatter (www.kalbermatter.ch)
//**********************************************************************

unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus,  ComCtrls, StdCtrls, ExtCtrls,  Buttons, Spin;

const
  DMXChannelsMax = 512;

type
  TForm1 = class(TForm)
    StatusBar: TStatusBar;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Panel1: TPanel;
    Help1: TMenuItem;
    About1: TMenuItem;
    Exit1: TMenuItem;
    ListView1: TListView;
    OpenBtn: TBitBtn;
    RefreshBtn: TBitBtn;
    CloseBtn: TBitBtn;
    QuitBtn: TBitBtn;
    Channel1: TTrackBar;
    Channel2: TTrackBar;
    Channel4: TTrackBar;
    Channel5: TTrackBar;
    Channel6: TTrackBar;
    Channel7: TTrackBar;
    Channel8: TTrackBar;
    Channel3: TTrackBar;
    Channel9: TTrackBar;
    Channel10: TTrackBar;
    Channel11: TTrackBar;
    Channel12: TTrackBar;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    StartEdit: TSpinEdit;
    Label13: TLabel;
    Timer1: TTimer;
    procedure FormShow(Sender: TObject);
    procedure RefreshBtnClick(Sender: TObject);
    procedure ListView1Click(Sender: TObject);
    procedure OpenBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure ChannelChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure QuitBtnClick(Sender: TObject);
  private
    DMX_Buf : Array[0..DMXChannelsMax] of byte;
  end;


var
  Form1: TForm1;


implementation

uses D2XXUnit;

{$R *.DFM}


procedure TForm1.RefreshBtnClick(Sender: TObject);
var i : Integer;
    ok : boolean;
    DeviceCount : DWord;
    LV : TListItem;
    aDevice_String : Array [1..50] of Char;
    SerialNo : String;

begin
  ListView1.Items.clear;
  FT_GetNumDevices(DeviceCount,Nil,FT_LIST_NUMBER_ONLY);                        // Get Number of Devices
  If DeviceCount > 0 then
  begin
    For I := 0 to DeviceCount-1 do
    Begin
      ok := FT_ListDevices(i,@aDevice_String,(FT_OPEN_BY_SERIAL_NUMBER or FT_LIST_BY_INDEX)) = FT_OK;
      SerialNo := aDevice_String;
      if ok then ok := FT_ListDevices(i,@aDevice_String,(FT_OPEN_BY_DESCRIPTION or FT_LIST_BY_INDEX)) = FT_OK;
      // Devices already in use return an error while getting serial number or device description
      // Therefore we display only devices which didn't return an error
      if ok then
      begin
        LV := ListView1.Items.Add;
        LV.Caption := IntToStr(I);
        LV.SubItems.Add(SerialNo);
        LV.SubItems.Add(aDevice_String);
      end;
    End;
  end;
  OpenBtn.Enabled := (Listview1.SelCount > 0);
end;

procedure TForm1.OpenBtnClick(Sender: TObject);
Var Index : Integer;
begin
  Index := StrToInt(ListView1.Items.Item[ListView1.Selected.Index].Caption);    // Check which Device we selected
  If FT_Open(Index,FT_Handle) = FT_OK then                                      // Try to open the Device
  Begin
    StatusBar.Panels[1].Text := 'Device '+IntToStr(Index)+' open';              // Display Open-Message in Status Bar
    ListView1.Enabled  := False;
    RefreshBtn.Enabled := False;
    OpenBtn.Enabled    := False;
    CloseBtn.Enabled   := True;
    FT_ResetDevice(FT_Handle);                                                  // Reset
    FT_SetTimeouts(FT_Handle,16,50);                                            // Define TX Timeout (RX is not important for this app)
    FT_SetBaudRate(FT_Handle,250000);                                           // Set the DMX Baudrate
    FT_SetDataCharacteristics(FT_Handle,FT_DATA_8,FT_STOP_2,FT_PARITY_NONE);    // Set DMX512 Framing Characteristics
    FT_SetFlowControl(FT_Handle,FT_FLOW_NONE,FT_XON_Value,FT_XOFF_Value);       // DMX512 needs no Handshaking
    Timer1.Enabled := True;                                                     // Restart the DMX Refresh
  End;
end;

procedure TForm1.ListView1Click(Sender: TObject);
begin
  OpenBtn.Enabled := (Listview1.SelCount > 0);
  if (Listview1.SelCount > 0) then
  begin
    ListView1.Selected := ListView1.Items[ListView1.Selected.Index];
    StatusBar.Panels[1].Text := ' Press Open';
  end;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var ok : boolean;
    Count : DWord;
begin
  Timer1.Enabled := False;                                                      // Stop DMX Refresh Timer
  ok := FT_SetBreakOn(FT_Handle)=FT_ok;                                         // Set the Reset condition
  if ok then ok := FT_SetBreakOff(FT_Handle) = FT_OK;                           // Remove the Reset condition
  if ok then ok := FT_Write(FT_Handle,@DMX_Buf,DMXChannelsMax+1,Count)=FT_ok;   // Send the Start-Byte and the 512 DMX512 channels
  if ok then Timer1.Enabled := True                                             // Restart the DMX Refresh Timer
        else CloseBtnClick(Self);                                               // If we get an error, we stop the transmission
end;

procedure TForm1.CloseBtnClick(Sender: TObject);
begin
  Timer1.Enabled := False;                                                      // Stop the DMX Refresh
  FT_Close(FT_Handle);                                                          // Close the USB Device
  FT_Handle := 0;                                                               // Reset the Handle
  RefreshBtn.Enabled := True;
  CloseBtn.Enabled := False;
  ListView1.Enabled := True;
  RefreshBtnClick(Self);
  StatusBar.Panels[1].Text := 'Please select a Device!';                        // Hint in the Status Bar
end;

procedure TForm1.ChannelChange(Sender: TObject);
var i,Pos : integer;
begin
  DMX_Buf[0] := 0;                                                              // Clear the Start-Byte (must be normaly zero!)
  for i := 1 to StartEdit.Value -1 do DMX_Buf[i] := 0;                          // Clear all channels up to the StartChannel
  for i := 0 to 11 do                                                           // Now we send the real channels
  begin
    Pos := (255 - TTrackBar(FindComponent('Channel'+IntToStr(i+1))).Position);  // Get the Slider Position
    DMX_Buf[StartEdit.Value+i] := Pos;                                          // Store the Slider Position in DMX Buffer
  end;
  for i := StartEdit.Value+12 to DMXChannelsMax do DMX_Buf[i] := 00;            // Fillup the remaining channels with 0;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  CloseBtnClick(Self);
  RefreshBtnClick(Self);
end;

procedure TForm1.QuitBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if FT_Handle <> 0 then FT_Close(FT_Handle);
  Exit;
end;

procedure TForm1.About1Click(Sender: TObject);
begin
  MessageDlg('  Demo for simple USB to DMX512 converter'+#13+
             '        based on the FTDI FT232BM chip'+#13+#13+
             '      (W) `2003 by Stefan Kalbermatter'+#13+#13+
             '               www.kalbermatter.ch'+#13, mtInformation,
             [mbOk], 0);
end;

end.
