//**********************************************************************
// Demo Application for simple USB to DMX512 Convertor based on the 
// FTDI Chip FT232BM. The device is controlled via FTDI's Direct Drivers
// V1.0 12.2003 Initial Version
//              (W) by Stefan Kalbermatter (www.kalbermatter.ch)
// V1.1 10.2004 DMX transmission is handled i separate thread
//              (W) by Stefan Kalbermatter (www.kalbermatter.ch)
//**********************************************************************

unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus,  ComCtrls, StdCtrls, ExtCtrls,  Buttons, Spin, mmSystem;

const
  DMXChannelsMax = 512;

type
  TSendThread = class(TThread)
  protected
    ok : Boolean;
    procedure Execute; override;
  public
    DMX_Run : Boolean;
    DMX_Buf : Array[0..DMXChannelsMax] of byte;
  end;

  TForm1 = class(TForm)
    StatusBar: TStatusBar;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Panel1: TPanel;
    Help1: TMenuItem;
    Exit1: TMenuItem;
    ListView1: TListView;
    OpenBtn: TBitBtn;
    RefreshBtn: TBitBtn;
    CloseBtn: TBitBtn;
    QuitBtn: TBitBtn;
    Channel1: TTrackBar;
    Channel2: TTrackBar;
    Channel4: TTrackBar;
    Channel5: TTrackBar;
    Channel6: TTrackBar;
    Channel7: TTrackBar;
    Channel8: TTrackBar;
    Channel3: TTrackBar;
    Channel9: TTrackBar;
    Channel10: TTrackBar;
    Channel11: TTrackBar;
    Channel12: TTrackBar;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    StartEdit: TSpinEdit;
    Label13: TLabel;
    procedure FormShow(Sender: TObject);
    procedure RefreshBtnClick(Sender: TObject);
    procedure ListView1Click(Sender: TObject);
    procedure OpenBtnClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure ChannelUpdate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure QuitBtnClick(Sender: TObject);
    procedure Help1Click(Sender: TObject);
  private
    SendThread : TSendThread;
    procedure ThreadStart;
    procedure ThreadStop;
    procedure ThreadStoping(Sender: TObject);
  end;



var
  Form1: TForm1;


implementation

uses D2XXUnit;

{$R *.DFM}

// Thread routines
procedure TSendThread.Execute;
var Count : DWord;
begin
  Repeat
    ok := FT_SetBreakOn(FT_Handle)=FT_ok;                                       // Set the Reset condition
    if ok then ok := FT_SetBreakOn(FT_Handle)=FT_ok;                            // Set the Reset condition (delay)
    if ok then ok := FT_SetBreakOff(FT_Handle) = FT_OK;                         // Remove the Reset condition
    if ok then ok := FT_Write(FT_Handle,@DMX_Buf,DMXChannelsMax+1,Count)=FT_ok; // Send the Start-Byte and the 512 DMX512 channels
    Sleep(4);                                                                   // Delay to allow transmission of chip internal buffer
  until not ok or Terminated;
end;

procedure TForm1.ThreadStart;
begin
  ThreadStop;                                                                   // possibly stop thread
  SendThread := TSendThread.Create(True);                                       // Suspended
  SendThread.OnTerminate := ThreadStoping;                                      // Call SendStop when terminating
  ChannelUpdate(Self);                                                          // Update DMX Channels
  SendThread.Resume;                                                            // Start DMX transmission
end;

procedure TForm1.ThreadStop;
begin
  if SendThread <> nil Then
  begin
    SendThread.Terminate;
    SendThread.WaitFor;
    SendThread.Free;
    SendThread := Nil;
  end;
end;

procedure TForm1.ThreadStoping(Sender: TObject);
begin
  FT_Close(FT_Handle);                                                          // Close the USB Device
  FT_Handle := 0;                                                               // Reset the Handle
  RefreshBtn.Enabled := True;
  CloseBtn.Enabled := False;
  ListView1.Enabled := True;
  RefreshBtnClick(Self);
  StatusBar.Panels[1].Text := 'Please select a Device!';                        // Hint in the Status Bar
end;

// GUI routines
procedure TForm1.RefreshBtnClick(Sender: TObject);
var i : Integer;
    ok : boolean;
    DeviceCount : DWord;
    LV : TListItem;
    aDevice_String : Array [1..50] of Char;
    SerialNo : String;
begin
  ListView1.Items.clear;
  FT_GetNumDevices(DeviceCount,Nil,FT_LIST_NUMBER_ONLY);                        // Get Number of Devices
  If DeviceCount > 0 then
  begin
    For I := 0 to DeviceCount-1 do
    Begin
      ok := FT_ListDevices(i,@aDevice_String,(FT_OPEN_BY_SERIAL_NUMBER or FT_LIST_BY_INDEX)) = FT_OK;
      SerialNo := aDevice_String;
      if ok then ok := FT_ListDevices(i,@aDevice_String,(FT_OPEN_BY_DESCRIPTION or FT_LIST_BY_INDEX)) = FT_OK;
      // Devices already in use return an error while getting serial number or device description
      // Therefore we display only devices which didn't return an error
      if ok then
      begin
        LV := ListView1.Items.Add;
        LV.Caption := IntToStr(I);
        LV.SubItems.Add(SerialNo);
        LV.SubItems.Add(aDevice_String);
        if i = 0 then LV.Selected := True;                                      // Automatically select first device
      end;
    End;
  end;
  ListView1Click(Self);
end;

procedure TForm1.OpenBtnClick(Sender: TObject);
Var Index : Integer;
begin
  Index := StrToInt(ListView1.Items.Item[ListView1.Selected.Index].Caption);    // Check which Device we selected
  If FT_Open(Index,FT_Handle) = FT_OK then                                      // Try to open the Device
  Begin
    StatusBar.Panels[1].Text := 'Device '+IntToStr(Index)+' open';              // Display Open-Message in Status Bar
    ListView1.Enabled  := False;
    RefreshBtn.Enabled := False;
    OpenBtn.Enabled    := False;
    CloseBtn.Enabled   := True;
    FT_ResetDevice(FT_Handle);                                                  // Reset
    FT_SetTimeouts(FT_Handle,16,50);                                            // Define TX Timeout (RX is not important for this app)
    FT_SetBaudRate(FT_Handle,250000);                                           // Set the DMX Baudrate
    FT_SetDataCharacteristics(FT_Handle,FT_DATA_8,FT_STOP_2,FT_PARITY_NONE);    // Set DMX512 Framing Characteristics
    FT_SetFlowControl(FT_Handle,FT_FLOW_NONE,FT_XON_Value,FT_XOFF_Value);       // DMX512 needs no Handshaking
    ThreadStart;
    CloseBtn.SetFocus;
  End;
end;

procedure TForm1.ListView1Click(Sender: TObject);
begin
  OpenBtn.Enabled := (Listview1.SelCount > 0);
  if (Listview1.SelCount > 0) then
  begin
    ListView1.Selected := ListView1.Items[ListView1.Selected.Index];
    StatusBar.Panels[1].Text := ' Press Open';
    OpenBtn.SetFocus;
  end;
end;

procedure TForm1.CloseBtnClick(Sender: TObject);
begin
  ThreadStop;
end;

procedure TForm1.ChannelUpdate(Sender: TObject);
var i,Pos : integer;
begin
  if SendThread <> Nil then With SendThread do
  begin
    DMX_Buf[0] := 0;                                                              // Clear the Start-Byte (must be normaly zero!)
    for i := 1 to StartEdit.Value -1 do DMX_Buf[i] := 0;                          // Clear all channels up to the StartChannel
    for i := 0 to 11 do                                                           // Now we send the real channels
    begin
      Pos := (255 - TTrackBar(FindComponent('Channel'+IntToStr(i+1))).Position);  // Get the Slider Position
      DMX_Buf[StartEdit.Value+i] := Pos;                                          // Store the Slider Position in DMX Buffer
    end;
    for i := StartEdit.Value+12 to DMXChannelsMax do DMX_Buf[i] := 00;            // Fillup the remaining channels with 0;
  end;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  ThreadStoping(Self);
  RefreshBtnClick(Self);
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ThreadStop;
  FT_Close(FT_Handle);
end;

procedure TForm1.Help1Click(Sender: TObject);
begin
  MessageDlg('Demo for simple USB to DMX512 converter (Threaded)'+#13+
             '            based on the FTDI FT232BM chip'+#13+#13+
             '          (W) `2004 by Stefan Kalbermatter'+#13+#13+
             '                   www.kalbermatter.ch'+#13, mtInformation,
             [mbOk], 0);

end;


procedure TForm1.QuitBtnClick(Sender: TObject);
begin
  Close;
end;


end.
